inherit GTK2.TextView;

object buffer;
object iter,start,end;
mapping tags=([]);
array taglist=({});

void create(mixed ... args) {
  ::create(@args);
  buffer=get_buffer();
  iter=buffer->get_iter_at_offset(0);
}

object insert_text(string text) {
  buffer->insert(iter,text,-1);
  start=buffer->get_iter_at_offset(0);
  start->set_offset(iter->get_offset()-sizeof(text));
  end=buffer->get_iter_at_offset(0);
  end->set_offset(iter->get_offset());
  foreach (taglist, string tag) {
    buffer->apply_tag(tags[tag],start,end);
  }
  taglist=({});
  return this_object();
}

object set_fg(string color) {
  string tag="fg-"+color;
  if (!has_index(tags,tag)) {
    tags[tag]=buffer->create_tag(tag,([ "foreground": color ]));
  }
  taglist+=({ tag });
  return this_object();
}

object set_bg(string color) {
  string tag="bg-"+color;
  if (!has_index(tags,tag)) {
    tags[tag]=buffer->create_tag(tag,([ "background": color ]));
  }
  taglist+=({ tag });
  return this_object();
}

object set_size(int size) {
  string tag="size-"+(string)size;
  if (!has_index(tags,tag)) {
    tags[tag]=buffer->create_tag(tag,([ "size": size*Pango.Scale ]));
  }
  taglist+=({ tag });
  return this_object();
}

object set_style(int style) {
  string tag="style-"+(string)style;
  if (!has_index(tags,tag)) {
    tags[tag]=buffer->create_tag(tag,([ "style": style ]));
  }
  taglist+=({ tag });
  return this_object();
}

object set_family(string family) {
  string tag="family-"+family;
  if (!has_index(tags,tag)) {
    tags[tag]=buffer->create_tag(tag,([ "family": family ]));
  }
  taglist+=({ tag });
  return this_object();
}

object set_wrap(int wrap) {
  string tag="wrap-"+(string)wrap;
  if (!has_index(tags,tag)) {
    tags[tag]=buffer->create_tag(tag,([ "wrap-mode": wrap ]));
  }
  taglist+=({ tag });
  return this_object();
}

object set_justify(int justify) {
  string tag="justify-"+(string)justify;
  if (!has_index(tags,tag)) {
    tags[tag]=buffer->create_tag(tag,([ "justification": justify ]));
  }
  taglist+=({ tag });
  return this_object();
}

object set_left_margin(int left) {
  string tag="left-margin-"+(string)left;
  if (!has_index(tags,tag)) {
    tags[tag]=buffer->create_tag(tag,([ "left-margin": left ]));
  }
  taglist+=({ tag });
  return this_object();
}

object set_right_margin(int right) {
  string tag="right-margin-"+(string)right;
  if (!has_index(tags,tag)) {
    tags[tag]=buffer->create_tag(tag,([ "right-margin": right ]));
  }
  taglist+=({ tag });
  return this_object();
}

object set_margins(int left, int right) {
  string tag="margins-"+string(left)+"-"+(string)right;
  if (!has_index(tags,tag)) {
    tags[tag]=buffer->create_tag(tag,([ "left-margin": left, "right-margin": right ]));
  }
  taglist+=({ tag });
  return this_object();
}

object set_strikethrough() {
  string tag="strikethrough";
  if (!has_index(tags,tag)) {
    tags[tag]=buffer->create_tag(tag,([ "strikethrough": 1 ]));
  }
  taglist+=({ tag });
  return this_object();
}

object set_underline() {
  string tag="underline";
  if (!has_index(tags,tag)) {
    tags[tag]=buffer->create_tag(tag,([ "underline": Pango.UnderlineSingle ]));
  }
  taglist+=({ tag });
  return this_object();
}

object set_indent(int indent) {
  string tag="indent-"+(string)indent;
  if (!has_index(tags,tag)) {
    tags[tag]=buffer->create_tag(tag,([ "indent": indent ]));
  }
  taglist+=({ tag });
  return this_object();
}

object set_superscript() {
  string tag="superscript";
  if (!has_index(tags,tag)) {
    tags[tag]=buffer->create_tag(tag,([ "rise": 10*Pango.Scale, "size": 8*Pango.Scale ]));
  }
  taglist+=({ tag });
  return this_object();
}

object set_subscript() {
  string tag="subscript";
  if (!has_index(tags,tag)) {
    tags[tag]=buffer->create_tag(tag,([ "rise": -10*Pango.Scale, "size": 8*Pango.Scale ]));
  }
  taglist+=({ tag });
  return this_object();
}
